/**
 * 可以考虑 JS API 暴露
 */
import _ from 'lodash';

export const getDpIdMapByDevInfo = (
  devInfo: DevInfo
): Record<string, DevInfo['schema'][number]> => {
  const dpIdMap = _.mapKeys(devInfo.schema, schemaInfo => schemaInfo.id);
  return dpIdMap;
};

export const getDpCodeMapByDevInfo = (
  devInfo: DevInfo
): Record<string, DevInfo['schema'][number]> => {
  const dpCodeMap = _.mapKeys(devInfo.schema, schemaInfo => schemaInfo.code);
  return dpCodeMap;
};

export const mapDpsMapToDpStateMap = (dpsMap: Record<string, any>, devInfo: DevInfo) => {
  const dpIdMap = getDpIdMapByDevInfo(devInfo);
  const dpStateMap = _.mapKeys(dpsMap, (__, dpId) => dpIdMap[dpId].code);
  return dpStateMap;
};

/**
 *
 * @param devInfo
 * @returns dpState 对象
 * ```
 * {
 *   switch: true,
 *   ...
 * }
 * ```
 */
export const getDpStateMapByDevInfo = (devInfo: DevInfo): Record<string, any> => {
  const { dps, schema } = devInfo;
  const dpStateMap = {};
  _.forEach(schema, schemaInfo => {
    dpStateMap[schemaInfo.code] = dps[schemaInfo.id];
  });
  return dpStateMap;
};

/**
 * @desc 将16进制的hsv转换成10进制的hsv
 * 范围为h(0-360) s(0-1000) v(0-1000)
 * @param {String} hsvStr - encoded hsvStr (hhhhssssvvvv)
 *
 * @return {Array} [h, s, v]
 *
 */
export const dpdataToHsv = (dpData: string): Array<any> => {
  if (!dpData || dpData.length !== 12) {
    return [0, 1000, 1000];
  }
  const b = dpData.match(/[a-z\d]{4}/gi) || [];
  return b.reduce((curr: number[], hex: string) => {
    curr.push(parseInt(hex, 16));
    return curr;
  }, []);
};

/**
 * @desc 将10进制的hsv转换成16进制的hhsssvvvv
 * 范围为h(0-360) s(0-1000) v(0-1000)
 * @param {Array} hsvArr - [h, s, v]
 *
 * @return {String} 'hhhhssssvvvv'
 *
 */
export const hsvToDpdata = (h: number, s: number, v: number): string => {
  let hue = h % 360;
  hue = hue > 0 ? hue : h;
  hue = hue < 0 ? 360 + hue : hue;

  return [hue, s, v].reduce((curr: string, next: number) => {
    let hex = parseInt(`${next}`, 10).toString(16);
    hex = format(hex, 4);
    return curr + hex;
  }, '');
};

// 位数填充
export const format = (value: string, len = 2) => {
  let v = `${value}`;
  if (v.length < len) {
    v = '0'.repeat(len - v.length) + v;
  } else {
    v = v.slice(0, len);
  }
  return v;
};

/**
 * @desc 将色温转化为rgb
 * 范围为2700~6500
 * 对应范围为1000~40000
 * @param {number} temperature - 5000
 *
 * @return {array} [r,g,b]
 *
 */

export const temperatureToRgb = (temperature: number): Array<any> => {
  let tempera = temperature;
  if (tempera < 1000) {
    tempera = 1000;
  }
  if (tempera > 40000) {
    tempera = 40000;
  }
  tempera /= 100;
  let red = 0;
  if (tempera <= 66) {
    red = 255;
  } else {
    red = tempera - 60;
    red = 329.698727446 * Math.pow(red, -0.1332047592);
    if (red < 0) {
      red = 0;
    } else if (red > 255) {
      red = 255;
    }
  }
  let green = 0;
  if (tempera <= 66) {
    green = tempera;
    green = 99.4708025861 * Math.log(green) - 161.1195681661;
    if (green < 0) {
      green = 0;
    }
    if (green > 255) {
      green = 255;
    }
  } else {
    green = tempera - 60;
    green = 288.1221695283 * Math.pow(green, -0.0755148492);
    if (green < 0) {
      green = 0;
    }
    if (green > 255) {
      green = 255;
    }
  }
  let blue = 0;
  if (tempera >= 66) {
    blue = 255;
  } else if (tempera <= 19) {
    blue = 0;
  } else {
    blue = tempera - 10;
    blue = 138.5177312231 * Math.log(blue) - 305.0447927307;
    if (blue < 0) {
      blue = 0;
    }
    if (blue > 255) {
      blue = 255;
    }
  }
  return [Math.floor(red), Math.floor(green), Math.floor(blue)];
};
